# Developer Wiki

## Introduction of DroneOA ROS

Drones are used in an unprecedented number of ways since the advent, with millions of people using it for agriculture, journalism, photography, courier services, etc. Seeing this opportunity, we propose to develop some solutions that make the development of drone applications easier (Including drone obstacle avoidance development) and make the drones safer at a lower altitude.

**DroneOA ROS** is a software package we have been working on to tackle this target. It provides:

- A set of unified interfaces for popular sensors and Mavlink based flight controllers
- Integrated obstacle avoidance features
- Development tool kit which consists of a UE4 based simulator, debug visualizer and debug tools
- A new simplified console command interface or a **console command "API"** for app developers
- An internal low-level control API (e.g. channel override, reset/switch communication link) for core feature developers.
- Application manager which allows installing new apps with simply a URL of its repository.
- A simplified Web-based UI API and a sample UI
- Easy to environment setup tools and prebuild and tested OS images of development/deploy environment.
- Full ROS Support and Open source

The goal of this project is to build an SDK that simplifies the development of automated drone and drone-related applications. It can also smooth the learning curve for new drone developers.

The team name **OASIS**, short for `Obstacle Avoidance with Spontaneous Itinerancy Strategies` is coming from one of our obstacle avoidance features, which allows multiple strategies (algorithm) running simultaneously. For example, both LIDAR and traditional camera with AI can be used to determine collision probabilities and/or path independently, which are then synthesized to provide a final decision.

We plan to deploy the project via drone communities, like the Ardupilot community. And potentially getting official support from the Ardupilot community. We will start to invite developers and people that are willing to learn drone development to try the system and join the development when the main control node and high priority features are finished. Community contributions and discussions are the core of this project in the long run.

### Modules And Current Progress:
![Top-Level Architecture](pics/HighArchitecture.png)

### Addon Development Strategies
- Check out [Contribute Page](Tips/Contribute.md)

### Hardware Support
- Check out [Hardware Page](Knowledge/Hardware.md)

### Developer Guideline and Knowledge

* [Setup Development Environment](Setup/DevSetupIndex.md)
  * [Setup Script](Setup/ENVSetupScript.md)
  * [Setup Manually](Setup/ManualENVSetup.md)
* [Test Code Change](Testing/DevTestingIndex.md)
  * [Launch With Launch Config](Testing/LaunchWithLaunchConfig.md)
  * [Use UE4 Simulator](Testing/TestWithUE4SIM.md)
* [Knowledge](Knowledge/DevKnowledgeIndex.md)
  * [Modules](Knowledge/Modules/ModulesIndex.md)
  * [Abbreviation](Knowledge/Abbreviation.md)
  * [Debug Features](Knowledge/DebugFeatures.md)
  * [Developer Workflow](Knowledge/DevWorkflow.md)
  * [Hardware Support](Knowledge/Hardware.md)
* [Tips & Tricks](Tips/TipsAndTricks.md)
* [Contribute Addons](Tips/Contribute.md)

## Credit
DroneOA Group 2020 :palm_tree:
