# Developer Workflow

You should always create an Issue first if you want to make changes.

## Start Working On A New Issue
* Read the requirements carefully, and ask the creator for anything unclear.
* When you decide to start:
  * Option 1: click the `Create Merge Request` button in the issue to create branch and merge request automatically
  * Option 2: create branch manually, and create merge request when you are ready.
    * The branch name should be: `[issue id]-[issue name]` replace any space with `-`
    * ```
      $ git checkout master
      $ git fetch
      $ git pull
      $ git checkout -b [issue id]-[issue name]
      ```
  * Mark Issue with label `DEVELOPMENT`, mark merge request with label `WIP` 

## Start A New Feature
1. Write a description or a memo for the new feature you want to implement. Including some use cases and the importance of this feature.
2. For concept proof stage, a detailed checklist will be enougth.
3. Create a QUESTION thread(Issue) to discuss with the team
4. If most people agree with adding the new feature, create Issue(s) according to the plan. **You are responsible to ensure the completion of the feature now**.

## Before committing
* Keep working branch up-to-date: (rebase only when mecessary)
  ```
  $ git checkout master
  $ git fetch
  $ git checkout [Work Branch]
  $ git merge master
  ```
* Run some style checks to save runners:
  * **cpplint** using command `cpplint --linelength=120 ./test/*.cpp ./src/*.cpp ./include/droneoa_ros/*.hpp ./src/OAC/*.cpp ./include/droneoa_ros/OAC/*.hpp ./src/Utils/*.cpp ./include/droneoa_ros/Utils/*.hpp`
  * catkinlint (if you modified the package info or makefile)
  * pylint (if you add new python script [Optional])
* Prepare you commit message:
  * Title: a brief summay of the purpose of the commmit
  * [Empty Line]
  * Issue ID
  * [Empty Line]
  * Content: any extra explaination of information you want to attach

## Maintain Merge Request
* A merge checklist should be generated automatically when creating merge request. Remember to ensure all items are completed
* Add desciption and explaination to the `Description` section of the merge request
* Check the pipeline to ensure your change did not break anything. Remember to check `haros` static analyze report to ensure you did not introduce any potential issues
* **Check Merge Request Settings**:
  * Set the correct label:
    * `WIP` for development or work in progress
    * `Ready For Verification` for ready for review and testing
    * `QUESTION` for discussion
    * `DO NOT MERGE` to enfore the merge request is not ready to merge or just a test
    * `PRIORITY` for high priority issues
    * `FEATURE` for new features, `BUG` for bug fix
    * etc.
  * You should request at least 1 review
  * Check squash commit

## Merge To Master
* Prerequirments for merge to master:
  * The pipeline has passed
  * No additional `haros` issues introduced
  * Code review approve received
  * Merge request checklist is all checked
* Merging:
  * Check the squash checkbox if you have more than 1 commits
  * Include enough detail in the commit message (e.g. issue ID and explaination of this change)
  * Resolve all outdated discussions
  * Wait for merge pipeline on master to success :)
* After merge:
  * Change label to `MERGED` for merge request and issue
