# The Command

**Note**: for entry `Pre-conditions & Checks`, FCU will always perform its own checks after receiving the command.

## General Supported Commands
- `CMD_CHMOD` or `cnc chmod <mode name>`
  - **DATA**: mod name
  - Change flight mode
  - Pre-conditions & Checks:
    - <mode name> is valid
- `CMD_ARM` or `cnc arm`
  - **DATA**: none
  - Arm the vehicle (also arm the throttle idle)
  - Pre-conditions & Checks:
    - Vehicle is Connected
    - All safety conditions are checked on FCU side.
- `CMD_TAKEOFF` or `cnc takeoff <altitude>`
  - **DATA**: float altitude
  - Takeoff to desired altitude
  - Pre-conditions & Checks:
    - Vehicle is Connected & Armed
    - Flight mode is Guided
    - <altitude> is below max altitude
- `CMD_LAND` or `cnc land`
  - **DATA**: none
  - Attempt to land
  - Pre-conditions & Checks:
    - All conditions are checked on FCU side
- `CMD_SET_MAX_VELOCITY` or `cnc velocity <new max velocity>`
  - **DATA**: float speed
  - Set the maximum velocity limitaion
  - Pre-conditions & Checks:
    - <new max velocity> is below max allowed velocity in the config
- `CMD_SET_YAW` or `cnc yaw <heading degree>`
  - **DATA**: float heading
  - Change the yaw / heading of the vehicle. Can be used to direct the sensors
  - Pre-conditions & Checks:
    - Flight mode is Guided
    - <heading degree> within 0-360 (Conversion will be added later)
- `CMD_GOTO_RELATIVE` or quik command like `! w <distance>`
  - **DATA**: float North axis dist, float East axis dist, float Altitude
  - Command the vehicle to a location `North axis x meter` and `East axis y meter` away from the current position
  - Pre-conditions & Checks:
    - Flight mode is Guided
- `CMD_GOTO_GLOBAL`
  - **DATA**: float Latitude, float Longitude, float Altitude
  - Command the vehicle to a location with GPS coordination
  - Pre-conditions & Checks:
    - Flight mode is Guided
- `CMD_GOTO_HEADING` or `cnc yaw <heading degree> <distance> <altitude>`
  - **DATA**: float heading, float distance, float Altitude
  - Command the vehicle to fly towards a specific heading until `distance` meter and at the same time climb to `altitude`.
  - `<distance>` and `<altitude>` are optional
  - Pre-conditions & Checks:
    - Flight mode is Guided
    - <heading degree> within 0-360 (Conversion will be added later)
    - <altitude> is below max altitude
- `CMD_CLIMB` or `cnc climb <delta altitude>`
  - **DATA**: float Delta Altitude
  - Climb delta altitude
  - Pre-conditions & Checks:
    - New altitude is below max allowed altitude
- `CMD_DESCEND` or `cnc decend <delta altitude>`
  - **DATA**: float Delta Altitude
  - Decend delta altitude
  - Pre-conditions & Checks:
    - New altitude is greater than 0 (AGL)

## Command Queue
**Basic grammar**:

Command = CMD, can be either `General Supported Commands` or `Command Specific Operations`.

- Mininum Command Queue:
  ```
  START <CMD> END
  ```
  Note: if a `delay` or `until` command is the **only** command in the queue, the runner will ignore the command and go the idle status. Since a single `delay` or `until` does not have any effect.
- Multiple Command:
  ```
  START <CMD> THEN <CMD> THEN ...... END
  ```
  Note: if a `delay` or `until` command is the **last** command in the queue, the runner will ignore the command and go the idle status. Since `delay` or `until` as the last command does not have any effect.

**Command Specific Operations**:
- `delay <time in ms>`:
  - Or `CMD_DELAY_MSEC,  DATA: uint32 time in ms`
  - The queue runner will wait for given time before roceed to the next command in the queue
- `until <mode> [data]`
  - Or `CMD_UNTIL,  DATA: mode (arrwp: arrive at way point; clrwp: clear all waypoints; ...)`
  - `until arrwp` wait until arrive at a waypoint.
    - The queue runner will wait until arrive at the next waypoint before roceed to the next command in the queue.
    - The way point comparison margin of error is defined by `GPS_COMPARE_DIFF_MAX`
  - `until clrwp` wait until clear of all waypoints in the queue.
    - The queue runner will wait until clear of all waypoints in the waypoint list before roceed to the next command in the queue
  - `until alt <operator> <altitude in m>`
    - The operator can be `eq`, `gt` or `lt` represents `equal`, `greater`, or `less than`
    - For `eq` the comparison margin of error is defined by `ALT_COMPARE_DIFF_MAX`
    - When the given condition is satisfied, the queue runner will proceed to the next command in the queue
- More to be added

### Generic Use Case Sequence For Command Queue

The use user use command queue to command the aircraft to takeoff, fly to somewhere (for example fly north 25 meters), then land.

Assume the obstacle avoidance (OA) function is disabled. More detail about OA related use cases, exceptions and alternatives will be discussed in OAC document. So the command queue the user would type would be something like:
`START cnc arm THEN delay 3000 THEN cnc takeoff 5 THEN until alt gt 4.5 THEN cnc yaw 0 25 THEN until arrwp THEN cnc land END`

For low level use case squence and exception of each involved commands, checkout [Single Command Use Cases](Developer/Knowledge/Modules/ConsoleManager.md) System component `Command Runner` is included to better illustrate the sequence and exception.

| APP/USER                            | IO/ConsoleManager                                                | DoneOA Main Node (Command Runner)                 | Drone                                                           |
|-------------------------------------|------------------------------------------------------------------|---------------------------------------------------|-----------------------------------------------------------------|
| 1. Typing in command queue as above |                                                                  |                                                   |                                                                 |
|                                     | 2. Parse the command queue and check for format and logic errors |                                                   |                                                                 |
|                                     | 3. Translate to internal formate and send to Runner              |                                                   |                                                                 |
|                                     |                                                                  | 4. Send arm motor command to drone                |                                                                 |
|                                     | 5. Display arm motor command sent message                        |                                                   |                                                                 |
|                                     |                                                                  |                                                   | 6. Arm the motor                                                |
|                                     |                                                                  | 7. Wait for 3000ms                                |                                                                 |
|                                     |                                                                  | 8. Send takeoff to 5 meter command to drone       |                                                                 |
|                                     | 9. Display takeoff command sent message                          |                                                   |                                                                 |
|                                     |                                                                  |                                                   | 10. Takeoff to 5 meter                                          |
|                                     |                                                                  | 11. Wait until altitude is greater than 4.5 meter |                                                                 |
|                                     |                                                                  |                                                   | 12. Altitude reached 4.5 meters                                 |
|                                     |                                                                  | 13. Send fly towards north for 25 meters command  |                                                                 |
|                                     | 14. Display fly to command sent message                          |                                                   |                                                                 |
|                                     |                                                                  |                                                   | 15. Start to fly to north for 25 meters                         |
|                                     |                                                                  | 16. Wait until the next waypoint is arrived       |                                                                 |
|                                     |                                                                  |                                                   | 17. Waypoint (North, 25m from origin, 5m AGL) arrived           |
|                                     |                                                                  | 18. Send land now command to drone                |                                                                 |
|                                     | 19. Display land command sent message                            |                                                   |                                                                 |
|                                     |                                                                  |                                                   | 20. Start landing procedure (Refer to landing command use case) |
|                                     | 21. Display execution Done message                               |                                                   |                                                                 |


**EXCEPTION for step 2**

| APP/USER | IO/ConsoleManager | DoneOA Main Node (Command Runner) | Drone |
|----------|-------------------|----------------|-------|
| | 2. Parse the command queue and check for format and logic errors. However, the is a format / logic error in the command queue | | |
| | 3. Command queue rejected. Display error message | | |

**EXCEPTION for step 4**

| APP/USER | IO/ConsoleManager | DoneOA Main Node (Command Runner) | Drone |
|----------|-------------------|----------------|-------|
| | | 4. Send arm motor command to drone. However the command fail to send due to hardware connection issue or channel congestion | |
| | 5. Display fail to send command message | | |
| | | 6. Try to send takeoff message. But the action is terminated due to the aircraft is not armed | |

**EXCEPTION for step 8**

| APP/USER | IO/ConsoleManager | DoneOA Main Node (Command Runner) | Drone |
|----------|-------------------|----------------|-------|
| | | 8. Send takeoff to 5 meter command to drone. However the command fail to send due to hardware connection issue or channel congestion | |
| | 9. Display fail to send command message | | |
| | | | 10. Aircraft disarm motor after 10 second because no takeoff command |

The `Command Runner` will still try to send follow up commands, but will have no effect because the drone is on the groud and disarmed.

**ALTERNATIVE for step 13**

Instead, achieved landing use case (at start location)

| APP/USER | IO/ConsoleManager | DoneOA Main Node (Command Runner) | Drone |
|----------|-------------------|----------------|-------|
| | | 13. Send fly towards north for 25 meters command. However the command fail to send due to hardware connection issue or channel congestion | |
| | 14.Display fail to send command message | | |
| | | 15. Wait until the next waypoint is arrived. However, since there are no waypoint been sent to the drone. The drone will never reach the next waypoint (There is a event). So it will wait until timeout | |
|                                     |                                                                  | 16. Send land now command to drone                |                                                                 |
|                                     | 17. Display land command sent message                            |                                                   |                                                                 |
|                                     |                                                                  |                                                   | 18. Start landing procedure (Refer to landing command use case) |
|                                     | 19. Display execution Done message                               |                                                   |                                                                 |

**ALTERNATIVE for step 18**

Instead, achieved return to land (RTL) use case.

Or user discover the issue and resend command or manual control the aircraft.

| APP/USER | IO/ConsoleManager | DoneOA Main Node (Command Runner) | Drone |
|----------|-------------------|----------------|-------|
| | | 18. Send land now command to drone. However the command fail to send due to hardware connection issue or channel congestion. | |
| | | | 19. Drone running out of battery |
| | 20. Display low battery RTL | | |
| | | | 21. Return to land at last takeoff location |
