# The Console Manager

## Introduction
**Console Input Manager** is currently the main IO module between the user (mission controller) and the droneoa system. As suggested by the name, Its job is to handle all the console input output communication and also some basic GUI commands.

There are mainly 2 big categories of commands:
- **Console Specific Commands**: which are commands that are only supported by the console. For example: `cnc info` which print out all important system status. This function is mainly used for debugging and serve no purpose in terms of other internal modules.
- **General Supported Commands**: which are commands also used by internal modules / apps (WIP). They are defined in COMMAND class and be parsed and executed by a shared standard internal parser. Command queue commands are also part of this General Supported Commands. (Checkout [Command wiki](Command.md) for more information)

In terms of the execution mode of the command, there are also 2 categories:
- **Instant Commands**: which are commands that itself can be treated as instant effect or single direction action, like sending a `cnc takeoff` command (Note: takeoff command itself does not wait for the vehicle to reach the desire altitude, it just send out an intension of a takeoff action). Usually, the order of the execution of Instant Commands doesn't affect the result, if there is no timed Commands in between.
- **timed Command**: which are commands that related to timing (e.g. delay, wait). They are only meaningfull in command queues. There are only 2 supported timed Command: `delay` (which delay and cause the runner to wait a certain amount of time) and `until` (which has modes.  For example: `until clrwp` will cause the runner to wait until all waypoints in the queue are cleared, before proceed the next command in the command queue).

The **Command Queue**:
- The command queue is the support for sending long queued sequence of commands or time related commands. The grammar will is discussed in the [Command wiki](Command.md).

The **GUI** support:
- The Console Input Manager listen to `droneoa/input_command` for string of commands from the GUI client. The received command will be validate and execute like normal console command(s) if no error in the grammar.

## Related Files
Currently the **Console Input Manager** is part of the Hardware interface (HWI).

Header: `droneoa_ros/include/droneoa_ros/HWI/ConsoleInputManager.hpp`

Source: `droneoa_ros/src/HWI/ConsoleInputManager.cpp`

## Console Specific Commands
- CNC
  - `info`: This is used to print important CNC info and status. For Debug use.
  - `quit`: Safely shutdown droneoa node.
- OAC
  - `on/off`: Enable/Disable the obstacle avoidance functionality (This will be moved to General Supported Commands)
  - `info`: This is used to print important OAC info and status. For Debug use.
- RSC
  - `chsrc <source name (e.g. ue4/ydlidar)>`: Change the data source of lidar module. For Debug use only. For normal operation, set in the config file.
  - `info`: This is used to print important RSC info and status. For Debug use.
  - `range <min(mm)> <max(mm)>`: Enable the range filter of the RSC module. For Debug use.
- Lidar
  - `chsrc <source name (e.g. ue4/ydlidar)>`: Change the data source of lidar module. For Debug use only. For normal operation, set in the config file.
  - `info`: This is used to print important Lidar info and status. For Debug use.

## Use cases (For 463):
**This section contains low-level unit use case** (Single command level). They are small cases with only one/no command involved and limited steps.

Check out Web GUI and Obstacle Avoidance module documentation for more complex real world use cases.

The purpose of this section is to demenstrate exceptions and alternative related to low-level hardware or software issues. Or exceptions that could happen with high "frequency" (e.g. on each command call). So when no duplicated exceptions will need to include in the complex use cases.

### Drone Update FCU Information and Data

| DroneWithSensors         | FCU ROS Client     | GUI | User/APP             |
|--------------------------|--------------------|-----|----------------------|
| 1. Flight controller unit (FCU) Data updated due to environment change or at constant rate |  |  |  |
| 2. Send update notification through Mavlink |  |  |  |
|                          | 3. Publish new info to corresponding topics using easier to process formate |  |  |
|                          |                            | 4. Update the GUI display with new information | |
|                          |                            |  | 5. Observe the new data |

**EXCEPTION for step 2**

| DroneWithSensors | FCU ROS Client | GUI | User/APP |
|------------------|----------------|-----|----------|
| 2. Send update notification. However the mavlink connection is lost or unstable. |  |  |  |
| 3. If there is no redundant communication channel, Drop this update. |  |  |  |

**ALTERNATIVE for step 2**

If the redundant communication channel is able to send the update successfully, we consoder the use case is achieved.

| DroneWithSensors | FCU ROS Client | GUI | User/APP |
|------------------|----------------|-----|----------|
| 2. Send update notification. However the mavlink connection is lost or unstable. |  |  |  |
| 3. If there is a redundant communication channel, try again by sending update through the redundant channel |  |  |  |

### Drone Update Sensor Information and Data

| DroneWithSensors         | Sensor ROS Client     | GUI | User/APP             |
|--------------------------|--------------------|-----|----------------------|
| 1. Sensor Data updated due to environment change or at constant rate |  |  |  |
| 2. Send update notification and data to OBC |  |  |  |
|                          | 3. Publish new info to corresponding topics and update local database/structure |  |  |
|                          |                            | 4. Update the GUI display with new information | |
|                          |                            |  | 5. Observe the new data |

**EXCEPTION for step 2**

| DroneWithSensors | FCU ROS Client | GUI | User/APP |
|------------------|----------------|-----|----------|
| 2. Send update notification. However connection is lost or unstable either due to hardware issues or channel congestion. |  |  |  |
| 3. This new data is dropped. |  |  |  |

### Generic Use Case Sequence For CNC Single Command

In the description of each specific commands, only pre-conditions, special exceptions and alternatives will be listed. Not deplicated use case sequence will be listed.

| APP/USER                                                                               | IO/ConsoleManager                          | Drone                                                 |
|----------------------------------------------------------------------------------------|--------------------------------------------|-------------------------------------------------------|
| 1. Type any single CNC command to the console, through the UI, or as string via ROS topic |                                            |                                                       |
|                                                                                        | 2. Parse Command, then sned to SYS to check for pre-conditions |                                                       |
|                                                                                        | 3. Send to drone via mavlink protocal         |                                                       |
|                                                                                        | 4. Display success sent notification and log the event |                                                       |
|                                                                                        |                                            | 5. Check for pre-condition on FCU side for some commands |
|                                                                                        |                                            | 6. Execute the corresponding  CNC command action         |

**EXCEPTION for step 2**

| APP/USER | IO/ConsoleManager | Drone |
|----------|-------------------|-------|
|  | 2. Parse Command, then sned to SYS to check for pre-conditions. However there are pre-condition unsatisfied in the provided data or command |  |
|  | 3. Display Error message, and log the error |  |

**EXCEPTION for step 3**

| APP/USER | IO/ConsoleManager | Drone |
|----------|-------------------|-------|
|  | 3. Send to drone via mavlink protocal. However the send action failed due to either hardware connection issue or channel congestion. |  |
|  | 4. Display Error message, and log the error |  |

**EXCEPTION for step 5**

| APP/USER | IO/ConsoleManager | Drone |
|----------|-------------------|-------|
|  |  | 5. Check for pre-condition on FCU side. However, some pre-conditions are not satisfied due to additional constrains or tranmission error |
|  |  | 6. Command reject, Aircraft stay in safe state |

**ALTERNATIVE for step 2**

Since user enabled `allow fall back to default` so consider using default value as achieved the use case.

| APP/USER | IO/ConsoleManager | Drone |
|----------|-------------------|-------|
|  | 2. Parse Command, then sned to SYS to check for pre-conditions. However there are pre-condition unsatisfied in the provided data or command |  |
|  | 3. For some command, if `allow fall back to default` is enabled and the command have a default value. Replace data with default safe value |  |
|  | 4. Display warning message, and log the event |  |

### Generic Use Case Sequence For request information (info)

| APP/USER                                                    | IO/ConsoleManager                              | Drone |
|-------------------------------------------------------------|------------------------------------------------|-------|
| 1. Request print status information about a specific module |                                                |       |
|                                                             | 2. Display the information on console and logs |       |

### Generic Use Case Sequence For change sensor data source (chsrc)

| APP/USER                                                            | IO/ConsoleManager                     | SYS/DroneOA Syatem                                   |
|---------------------------------------------------------------------|---------------------------------------|------------------------------------------------------|
| 1. Request change data source of a sensor module to a specified source (e.g. YDLidar real data --> UE4 SIM data) |                                       |                                                      |
|                                                                     | 2. Parse command and check for format |                                                      |
|                                                                     |                                       | 3. Listen to the new specified data topic |

**EXCEPTION for step 2**

| APP/USER                                                    | IO/ConsoleManager                              | Drone |
|-------------------------------------------------------------|------------------------------------------------|-------|
|                                                             | 2. Parse command and check for format. However the format is invalid or the specified source does not exist |  |
|                                                             | 3. Display error message and stay on the original data source |  |

### Use Case Sequence For change RSC range (RSC range)

| APP/USER                                                                               | IO/ConsoleManager                     | Depth Camera Interface                                                                     | GUI                                                        |
|----------------------------------------------------------------------------------------|---------------------------------------|--------------------------------------------------------------------------------------------|------------------------------------------------------------|
| 1. Request change the range of the depth camera (e.g. to only show data from 2m to 4m) |                                       |                                                                                            |                                                            |
|                                                                                        | 2. Parse command and check for format |                                                                                            |                                                            |
|                                                                                        |                                       | 3. Filter data and only keep data within the desired range from the next sensor dataupdate |                                                            |
|                                                                                        |                                       |                                                                                            | 4. Display filtered depth camera data instead of full data |

**EXCEPTION for step 2**

| APP/USER                                                                               | IO/ConsoleManager                     | Depth Camera Interface                                                                     | GUI                                                        |
|----------------------------------------------------------------------------------------|---------------------------------------|--------------------------------------------------------------------------------------------|------------------------------------------------------------|
|                                                                                        | 2. Parse command and check for format. However the format is invalid or the range is invalid |                                                                                            |
|  | 3. Display error message, range setting is not changed |  |

### Use Case Sequence For switch on Obstacle avoidance module (OAC on)

| APP/USER | IO/ConsoleManager | Obstacle Avoidance Controller | GUI | Drone |
|----------|-------------------|-------------------------------|-----|-------|
| 1. Request turn on the integrated obstacle avoidance module |                                                         |                                                            |                                          |                                                                                                               |
|                                                             | 2. Parse command and send  to obstacle avoidance module |                                                            |                                          |                                                                                                               |
|                                                             |                                                         | 3. Resume the obstacle avoidance planning and voting cycle |                                          |                                                                                                               |
|                                                             |                                                         |                                                            |                                          | 4. The drone will automatically avoid obstacle and plan path based on the mission given by the controller/App |
|                                                             |                                                         |                                                            | 5. Display new obstacle avoidance status |                                                                                                               |

**EXCEPTION for step 2**

| APP/USER | IO/ConsoleManager | Obstacle Avoidance Controller | GUI | Drone |
|----------|-------------------|-------------------------------|-----|-------|
|| 2. Parse command. However the command is invalid. ||||
|| 3. Display error message. Nothing is changed ||||

**ALTERNATIVE for step 3**

| APP/USER | IO/ConsoleManager | Obstacle Avoidance Controller | GUI | Drone |
|----------|-------------------|-------------------------------|-----|-------|
| | | 3. Resume the obstacle avoidance planning and voting cycle. However the obstacle avoidance planner is already running | | |

Nothing happened since we already achieved the desired use case

**EXCEPTION & ALTERNATIVE after step 4**
Detailed exception and alternatives will be given in OAC use case documents, which has complex use cases.

### Use Case Sequence For switch off Obstacle avoidance module (OAC off)

| APP/USER | IO/ConsoleManager | Obstacle Avoidance Controller | GUI | Drone |
|----------|-------------------|-------------------------------|-----|-------|
| 1. Request turn off the integrated obstacle avoidance module |                                                         |                                                           |                                          |                                                                                                  |
|                                                              | 2. Parse command and send  to obstacle avoidance module |                                                           |                                          |                                                                                                  |
|                                                              |                                                         | 3. Pause the obstacle avoidance planning and voting cycle |                                          |                                                                                                  |
|                                                              |                                                         |                                                           |                                          | 4. The drone will execute the mission given by the controller/App without caring about obstacles |
|                                                              |                                                         |                                                           | 5. Display new obstacle avoidance status |                                                                                                  |

**EXCEPTION for step 2**

| APP/USER | IO/ConsoleManager | Obstacle Avoidance Controller | GUI | Drone |
|----------|-------------------|-------------------------------|-----|-------|
|| 2. Parse command. However the command is invalid. ||||
|| 3. Display error message. Nothing is changed ||||

**ALTERNATIVE for step 3**

| APP/USER | IO/ConsoleManager | Obstacle Avoidance Controller | GUI | Drone |
|----------|-------------------|-------------------------------|-----|-------|
| | | 3. Pause the obstacle avoidance planning and voting cycle. However the obstacle avoidance planner is already stopped | | |

Nothing happened since we already achieved the desired use case
