# Obstacle Avoidance Controller

## Introduction & Deisgn Requiements
The main obstacle avoidance controller (OAC) is running on the main thread. It controls and manages algorithms. The OAC also makes decisions on which result to execute, then call the runner to execute command queue(s). Such an important component needs a well-designed architecture. Some functional requirements need to be satisfied when designing the architecture of the main controller.
1. **Control run cycle of algorithms.** The controller needs to control the basic stages of an algorithm. For example, when an algorithm instance can safely take data from sensors; when to start an emulation (or path calculation); when to output the result for evaluation (for choosing which algorithm to be executed based on for example confidence of path planning result).
2. **Select which algorithm to enable.** The controller should be able to disable a certain algorithm when its precondition is not satisfied or some exception happened.
3. **Adding in new algorithms easily.** The architecture should allow adding additional algorithms easily as either a new composited instance or a separate thread. So, the developer that using this system as a base system can easily add and test his own algorithm implementation.
4. **Fail safety.** The controller should be able to handle safety-related exceptions or call correct safety handlers to handle exceptions. For example, when GPS/localization signal is unstable or missing, safety should engage to give a warning then set the drone to land or hold altitude.
5. **Does NOT introduce too much overhead.** The controller affects the runtime efficiency of an algorithm or control command execution. It shouldn’t introduce too much unnecessary overhead in addition to algorithm runtime and communication delay. (Except for some very simple collision avoidance algorithms which need to be manually slow down to prevent communication congestion)
6. **Allow extra data or command (for some algorithm)** ** The controller can accept extra command or data and react based on it easily. For example, a developer wants to add in an RTT based path planning algorithm, but it takes 10 seconds to run which is way longer than other algorithms. Then the algorithm can send extra data to inform the controller it will take multiple cycles to calculate. So, the controller will not wait (to allow essential algorithms so the drone can still move) for it or kick it out (due to unresponsive).
7. **Pause and resume.** Part of the cycle control but be more specific. The controller needs to be able to pause and resume the control cycle. This is very important for something like resetting the system when collision avoidance safety is triggered (To distinguish from obstacle avoidance. Collision
avoidance will bring the drone to a full stop when an obstacle is very close and most path planning algorithms cannot react effectively or quickly enough).
8. **Easy to debug and implement.** Since this a platform or tool kit for drone developers. The controller needs to be easy to debug and dialog. Specifically for the capstone project, the team is small and we don’t have too much time to implement a very complex system, so the design needs to be easy to implement and maintain. I know some complex fully concurrent design will slightly reduce the
overhead.
**Note:** ** is optional

## Architecture
### Current Implementation: Self-Managed State Machine:
In this solution, the controller is more like a state machine running at a constant tick rate. The controller itself is on the main thread. Each algorithm instance will run on its own thread. There is a queue in the controller to track enabled algorithms and their priority (or order if sequential only). There will be 7 main states:
- **PAUSED**: state machine paused or used to reset controller when for example a collision-avoidance brake action is triggered.
- **INIT**: cleanup and initialize the state machine. The next step is to signal algorithms to collect data from sensors.
- **COLLECTED**: all sensor data is collected. The enabled algorithm list is filtered to kick out algorithms with insufficient precondition(s). The next step is to do path planning based on collected data.
- **PLANNED**: path is planned. Data (specific data like confidence), and command queue (sequence of commands to flight controller) are generated. The next step is to compare results and choose one or a combination to execute.
- **EXECUTED**: the chosen command queue is executed (based on confidence, priority, and previous action). The next step is to start over from INIT.
- **ABORT**: a critical exception happened and cause the main controller to terminate any currently executing command queue(s). The next step is to start over from INIT.
- **SAFETY**: a fatal exception happened which cause the main controller to terminate completely and the safety handler will be executed.
The main thread will switch between states at a constant rate or when an event happens (e.g. SLAM map updated, GPS unstable; No valid algorithm enabled, etc.). A slow algorithm can inform the controller to skip a cycle without affecting its confidence and priority. [Figure 1.]

![State Machine](pics/StateMachine.png)

### Potential Better Design With Addon Manager: Network like (Client-server)
The potential Better Design With Addon Manager is a network alike client-and-server design. In this design, each algorithm can be a separate ROS node (different process) or separate thread. They accept requests from the controller and return a future type result to the controller, then fill with result content later. For example: we have 3 algorithms enabled. The controller sends a request for a new path to each algorithm client through a broadcast topic when the map (from SLAM) is changed or at a constant rate (some basic algorithms use raw data instead of a map). The request includes a time mark to indicate which version of map the algorithm should obtain from the hardware interface OR a full copy of the map data. The algorithm returns a future as a result carrier immediately once receives the request, then populate with command and data when the calculation is done. Similar to state machine design, if an algorithm is unresponsive and did not send a command for extending execute for multiple cycles (put in future container), it will be kicked out from the valid algorithm list until reset.

![Client Server](pics/ClientServer.png)

## Detailed Information
- [Use Cases](OACUCs.md)

## Credit
DroneOA Group 2020 :palm_tree:
